<?php
namespace FreePBX\modules;

use BMO;
use FreePBX\modules\Pt1coutpanel\lib\AmigoDemons;
use FreePBX\modules\Pt1coutpanel\lib\LicenseWorker;
use FreePBX_Helpers;
use PDO;

class Pt1coutpanel extends FreePBX_Helpers implements BMO
{
    public $freepbx;

    public function __construct($freepbx = null)
    {
        parent::__construct($freepbx);
        if ($freepbx === null) {
            throw new Exception('Not given a FreePBX Object');
        }
        $this->freepbx = $freepbx;
        $this->db      = $freepbx->Database;
    }

    public function install()
    {
    }

    public function uninstall()
    {
    }

    public function backup()
    {
    }

    public function restore($backup)
    {
    }

    public function doConfigPageInit($page)
    {
    }

    /**
     * Add execution permission
     * @return array
     */
    public function chownFreePBX(){
        $binDir = \FreePBX::Config()->get('AMPBIN');
        $modulebindir = "{$binDir}/pt1coutpanel";
        $files = array();
        $files[] = array('type' => 'file',
                         'path' => $modulebindir,
                         'perms' => 0755);
        return $files;
    }
    /**
     * Start FreePBX for fwconsole hook
     *
     * @param object $output The output object.
     */
    public function startFreePbx($output = null)
    {
        $amigo = new AmigoDemons();
        $amigo->startAllServices();
    }

    /**
     * Включение HOOK на запись конфигов
     *
     * @param $conf
     */
    public function writeConfig($conf)
    {
        $this->FreePBX->WriteConfig($conf);
        if ($this->FreePBX->astman->connected() && ! $this->FreePBX->astman->mod_loaded("cdr_manager")) {
            exec(fpbx_which('asterisk') . " -rx 'module load cdr_manager'");
        }
    }

    /**
     * Переопределение параметров в конфигурационных файлах
     *
     * @return mixed
     */
    public function genConfig()
    {
        // Сформируем конфигурационные файлы для демонов
        $amigo = new AmigoDemons();
        // Добавим в крон демоны панели
        $amigo->updateCronTasks();

        // Добавим строчки в CDR manager
        $cdr_manager             = $this->FreePBX->LoadConfig->getConfig('cdr_manager.conf');
        $cdr_manager['general']  = [
            'enabled=yes',
        ];
        $cdr_manager['mappings'] = [
            'recordingfile=recordingfile',
            'linkedid=linkedid',
        ];

        $conf['cdr_manager.conf'] = $cdr_manager;

        // Добавим строчки в manager_additional.conf
        $manager_additional = $this->FreePBX->LoadConfig->getConfig('manager_additional.conf');
        $config             = $this->getConfig();
        $arr_params         = [
            'AgentCalled',
            'AttendedTransfer',
            'BlindTransfer',
            'Bridge',
            'BridgeEnter',
            'BridgeLeave',
            'BridgeMerge',
            'Cdr',
            'ChanSpyStart',
            'ContactStatus',
            'ContactStatusDetail',
            'Dial',
            'DialBegin',
            'DialEnd',
            'ExtensionStatus',
            'Hangup',
            'Hold',
            'Masquerade',
            'MeetmeEnd',
            'MeetmeJoin',
            'MeetmeLeave',
            'MeetmeTalking',
            'MusicOnHold',
            'MusicOnHoldStart',
            'NewCallerid',
            'Newchannel',
            'Newstate',
            'OriginateResponse',
            'ParkedCall',
            'ParkedCallGiveUp',
            'ParkedCallTimeOut',
            'UnParkedCall',
            'Unhold',
            'PeerEntry',
            'Pickup',
            'Rename',
            'UserEvent',
            'DataGetTree',
            'QueueCallerJoin',
            'QueueCallerLeave',
        ];

        $manager_additional[$config['ami_user']] = [
            'secret=' . $config['ami_password'],
            'deny=0.0.0.0/0.0.0.0',
            'permit=127.0.0.1/255.255.255.255',
            'read=agent,call,cdr,user',
            'write=system,call,originate,reporting',
            'eventfilter=!UserEvent: CdrConnector',
            'eventfilter=Event: (' . implode('|', $arr_params) . ')',
        ];

        $conf['manager_additional.conf'] = $manager_additional;

        return $conf;
    }

    /**
     * Получение данных из базы в форму
     *
     * @return |null
     */
    public function getConfig($var = null, $id = 'noid')
    {
        $sql = 'SELECT * FROM pt1coutpanel';
        $sth = $this->db->prepare($sql);
        $sth->execute();
        $data = $sth->fetch(PDO::FETCH_ASSOC);

        return $data;
    }

    /**
     * Добавляет кнопки на форму модуля
     *
     * @param $request
     *
     * @return array
     */
    public function getActionBar($request)
    {
        $buttons = [];
        switch ($request['display']) {
            case 'pt1coutpanel':
                $buttons = [
                    'submit' => [
                        'name'   => 'submit',
                        'id'     => 'submit',
                        'value'  => _('Submit'),
                        'hidden' => '',
                    ],
                ];
                break;
        }

        return $buttons;
    }

    /**
     * Ajax Request for BMO
     *
     * @param string $command [description]
     * @param [type] $setting [description]
     *
     * @return bool
     */
    public function ajaxRequest($command, &$setting)
    {
        switch ($command) {
            case 'edit':
            case 'check':
                return true;
            default:
                return false;
        }
    }

    /**
     * Ajax Module for BMO
     */
    public function ajaxHandler()
    {
        switch ($_REQUEST['command']) {
            case 'edit':
                return $this->saveConfig($_POST);
                break;
            case 'check':
                return $this->checkAmigoWorkers();
                break;
            default:
                return false;
        }
    }

    /**
     * Сохранение данных из формы в базу данных
     *
     * @param $post
     *
     * @return |null
     */
    public function saveConfig($post)
    {
        $errors        = [];
        $needReload    = false;
        $oldSettings   = $this->getConfig();
        $licenseWorker = new LicenseWorker();

        // Если переданы регистрационные данные, попробуем получить тириал
        if ( empty($post['saaskey'])
            && !empty($post['companyname'])
            && !empty($post['contact'])
            && !empty($post['email'])
            && !empty($post['telefone'])
        ) {
            $newLicenseKey = $licenseWorker->getTrialLicense($post);
            if (strlen($newLicenseKey) === 28
                && strpos($newLicenseKey, 'MIKO-') === 0) {
                $post['saaskey'] = $newLicenseKey;
                $needReload      = true;
            } else {
                // Не удалось получить триальную лицензию, попробуем вывести корректное сообщение об ошибке
                $errors[] = $licenseWorker->translateLicenseErrorMessage($newLicenseKey);
            }
        }

        // Добавим нужные триальные лицензии для модуля
        if (!empty($post['saaskey'])
            && $oldSettings['saaskey'] !== $post['saaskey']
            && strlen($post['saaskey']) === 28
            && strpos($post['saaskey'], 'MIKO-') === 0

        ) {
            $licenseWorker->changeLicenseKey($post['saaskey']);
            $licenseWorker->addTrial(3);
            $licenseWorker->addTrial(8);
        }

        // Активируем купон
        if (!empty($post['saaskey'])
            && !empty($post['coupon'])
            && strlen($post['saaskey']) === 28
            && strpos($post['saaskey'], 'MIKO-') === 0
            && strlen($post['coupon']) === 31
            && strpos($post['coupon'], 'MIKOUPD-') === 0
        ) {
            $result = $licenseWorker->activateCoupon($post['coupon']);
            if ($result !== true) {
                $errors[] = $licenseWorker->translateLicenseErrorMessage($result);
            } else {
                $needReload = true;
            }
        }

        // Сохраним настройки из формы
        foreach ($post as $key => $value) {
            switch ($key) {
                case 'server1chost':
                case 'server1cport':
                case 'login':
                case 'password':
                case 'database':
                case 'web_service_mode':
                case 'saaskey':
                    $sql           = "UPDATE pt1coutpanel SET `{$key}` = ?";
                    $prepStatement = $this->db->prepare($sql);
                    $prepStatement->execute([trim($value)]);
                    break;
                case 'debug_mode':
                    $sql           = "UPDATE pt1coutpanel SET `{$key}` = ?";
                    $prepStatement = $this->db->prepare($sql);
                    $result        = $value === 'on';
                    $prepStatement->execute([$result]);
                    break;
                default:
            }
        }

        // Включим модуль, если все нужные параметры заполнены
        $sql           = 'UPDATE pt1coutpanel SET `enable` = ?';
        $prepStatement = $this->db->prepare($sql);
        if (!empty($post['saaskey'])) {
            if ($post['web_service_mode'] === '0'
                || (
                    !empty($post['server1chost'])
                    && !empty($post['server1cport'])
                    && !empty($post['login'])
                    && !empty($post['password'])
                    && !empty($post['database'])
                )
            ) {
                $prepStatement->execute([true]);
            }
        } else {
            $prepStatement->execute([false]);
        }

        // Перезапустим сервисы
        $amigo = new AmigoDemons($this);
        $amigo->startAllServices(true);

        return [
            'status'      => count($errors) === 0,
            'messages'    => $errors,
            'need_reload' => $needReload,
        ];
    }

    /**
     * Возвращаем статусы модулей по AJAX запросу из формы модуля
     */
    private function checkAmigoWorkers()
    {
        // Перезапустим сервисы
        $amigo = new AmigoDemons($this);

        return $amigo->test();
    }

    /**
     * Отрисовать страницу модуля
     */
    public function showPage()
    {
        $data_value = $this->getConfig();

        return load_view(__DIR__ . '/views/settings.php', ['data_value' => $data_value]);
    }

}