<?php

namespace FreePBX\modules\Pt1coutpanel\lib;

use FreePBX;

class AmigoDemons
{
    public $dirs;
    public $module_settings;
    public $freepbx;
    public $webuser;
    public $webgroup;

    public function __construct(FreePBX\modules\Pt1coutpanel $pt1c = null)
    {
        if ($pt1c === null) {
            $pt1c = FreePBX::Pt1coutpanel();
        }
        $this->module_settings = $pt1c->getConfig();
        $this->freepbx         = $pt1c->freepbx;
        $this->webuser         = $this->freepbx->Config->get('AMPASTERISKUSER');
        $this->webgroup        = $this->freepbx->Config->get('AMPASTERISKGROUP');
        $this->dirs            = $this->getModuleDirs();
    }

    /**
     * Подготавливает директории для хранения конфигов и логов модуля
     *
     * @return array
     * @throws \Exception
     */
    private function getModuleDirs()
    {
        // binDir
        $binDir = $this->freepbx->Config->get('AMPBIN');
        if (isset ($binDir)) {
            $binDir = "$binDir/pt1coutpanel";
        } else {
            $binDir = '/var/lib/asterisk/bin/pt1coutpanel';
        }
        Util::createFolder($binDir, $this->webuser, $this->webgroup);


        // confDir
        $etcDir = $this->freepbx->Config->get('ASTETCDIR');
        if (isset ($etcDir)) {
            $confDir = "$etcDir/pt1coutpanel";
        } else {
            $confDir = '/etc/pt1coutpanel';
        }
        Util::createFolder($confDir, $this->webuser, $this->webgroup);

        // spoolDir
        $spoolDir = $this->freepbx->Config->get('ASTSPOOLDIR');
        if (isset ($spoolDir)) {
            $spoolDir = "$spoolDir/pt1coutpanel";
        } else {
            $spoolDir = '/var/spool/asterisk/pt1coutpanel';
        }
        Util::createFolder($spoolDir, $this->webuser, $this->webgroup);

        // logDir
        $logDir = $this->freepbx->Config->get('ASTLOGDIR');
        if (isset ($logDir)) {
            $logDir = "$logDir/pt1coutpanel";
        } else {
            $logDir = '/var/log/asterisk/pt1coutpanel';
        }
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        // pid
        $pidDir = '/var/run/asterisk/pt1coutpanel';
        Util::createFolder($pidDir, $this->webuser, $this->webgroup);

        //filesDir
        $filesDir = $this->freepbx->Config->get('ASTSPOOLDIR');
        if (isset ($filesDir)) {
            $filesDir = "$filesDir/pt1coutpanel/files";
        } else {
            $filesDir = '/var/spool/asterisk/pt1coutpanel/files';
        }
        Util::createFolder($filesDir, $this->webuser, $this->webgroup);

        return [
            'logDir'   => $logDir,
            'spoolDir' => $spoolDir,
            'confDir'  => $confDir,
            'pidDir'   => $pidDir,
            'binDir'   => $binDir,
            'filesDir' => $filesDir,
        ];
    }

    /**
     * Создает файл настройки очереди - nats.conf
     */
    private function generateNatsConf()
    {
        $sessionsDir = "{$this->dirs['spoolDir']}/sessions";
        Util::createFolder($sessionsDir, $this->webuser, $this->webgroup);

        $logDir = "{$this->dirs['logDir']}/gnats";
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        $pid_file = "{$this->dirs['pidDir']}/gnatsd-cti.pid";
        $settings = [
            'port'             => '4222',
            'http_port'        => '8222',
            'debug'            => 'false',
            'trace'            => 'false',
            'logtime'          => 'true',
            'pid_file'         => $pid_file,
            'max_connections'  => '1000',
            'max_payload'      => '1000000',
            'max_control_line' => '512',
            'sessions_path'    => $sessionsDir,
            'log_file'         => "{$logDir}/gnatsd.log",

        ];
        $config   = '';
        foreach ($settings as $key => $val) {
            $config .= "{$key}: {$val} \n";
        }

        file_put_contents("{$sessionsDir}/license.key", $this->module_settings['saaskey']);
        file_put_contents($this->dirs['confDir'] . '/nats.conf', $config);
    }

    /**
     * Создает файл настроек для демона monitord
     */
    private function generateMonitordConf()
    {

        $logDir = "{$this->dirs['logDir']}/monitord";
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        $settings_monitor = [
            'mq'        => [
                'host' => '127.0.0.1',
                'port' => '4222',
            ],
            'log_level' => $this->module_settings['debug_mode'] === '1' ? 6 : 2,
            'log_path'  => $logDir,
            'work_dir'  => $this->dirs['spoolDir'],
            'period'    => 30,
            'daemons'   => [
                [
                    'path'    => "{$this->dirs['binDir']}/amid",
                    'args'    => "-c {$this->dirs['confDir']}/ami.json",
                    'subject' => 'daemon.asterisk.ping',
                ],
                [
                    'path'    => "{$this->dirs['binDir']}/crmd",
                    'args'    => "-c {$this->dirs['confDir']}/crm.json",
                    'subject' => 'daemon.1c.ping',
                ],
                [
                    'path'    => "{$this->dirs['binDir']}/authd",
                    'args'    => "-c {$this->dirs['confDir']}/auth.json",
                    'subject' => 'daemon.auth.ping',
                ],
            ],
        ];
        file_put_contents("{$this->dirs['confDir']}/monitord.json",
            json_encode($settings_monitor, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
        file_put_contents("{$this->dirs['spoolDir']}/auth.hash", 'd41d8cd98f00b204e9800998ecf8427e');
    }

    /**
     * Создает файл настроек для демона crmd
     * Который отвечает за взаимодействие с CRM
     */
    private function generateCrmdConf()
    {
        $logDir = "{$this->dirs['logDir']}/crmd";
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        $settings_crm = [
            'mq'             => [
                'host' => '127.0.0.1',
                'port' => '4222',
            ],
            'log_level'      => $this->module_settings['debug_mode'] === '1' ? 6 : 2,
            'log_path'       => $logDir,
            'cleanup_period' => 10,  // Cache of links cleanup period.
        ];
        if ($this->module_settings['web_service_mode'] === '1') {
            $settings_crm['wsdl'] = [
                'host'       => $this->module_settings['server1chost'],
                'port'       => $this->module_settings['server1cport'],
                'login'      => $this->module_settings['login'],
                'password'   => $this->module_settings['password'],
                'url'        => "/{$this->module_settings['database']}/ws/miko_crm_api.1cws",
                'keep-alive' => 3000,
                'timeout'    => 10,
            ];
        } else {
            $settings_crm['long_poll'] = [
                'port'               => '8224',
                'event_time_to_live' => 10,
            ];
        }

        file_put_contents("{$this->dirs['confDir']}/crm.json",
            json_encode($settings_crm, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }

    /**
     * Создает файл настроек для демона authd
     */
    private function generateAuthdConf()
    {
        $logDir = "{$this->dirs['logDir']}/authd";
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        $settings_auth = [
            'log_level' => $this->module_settings['debug_mode'] === '1' ? 6 : 2,
            'log_path'  => $logDir,
            'mq'        => [
                'host' => '127.0.0.1',
                'port' => '4222',
            ],
            'auth'      => [
                'authorization'   => 'os',
                'settings_dir'    => '',
                'settings_source' => 'crm',
                'askozia_support' => false,
            ],
        ];
        file_put_contents("{$this->dirs['confDir']}/auth.json",
            json_encode($settings_auth, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }

    /**
     * Создает файл настроек для демона amid
     * подписывается на AMI события, обрабатывает их и отправляет в очередь
     */
    private function generateAmidConf()
    {

        $logDir = "{$this->dirs['logDir']}/amid";
        Util::createFolder($logDir, $this->webuser, $this->webgroup);

        $settings_amid = [
            'pbx'=> 'FreePBX',
            'originate'=> [
                'default_context'=> 'from-internal',
                'transfer_context'=> 'from-internal-xfer',
                'multiple_registration_support'=> false
            ],
            'mq'        => [
                'host' => '127.0.0.1',
                'port' => '4222',
            ],
            'log_level' => $this->module_settings['debug_mode'] === '1' ? 6 : 2,
            'log_path'  => $logDir,
            'ami'       => [
                'user'     => $this->module_settings['ami_user'],
                'password' => $this->module_settings['ami_password'],
                'host'     => '127.0.0.1',
                'port'     => (string)($this->freepbx->Config->get('ASTMANAGERPORT')),
            ],
            'database'  => [
                'path' => "{$this->dirs['spoolDir']}/history.db",
            ],
            'http'      => [
                'port'  => '8000',
                'limit' => 20,
            ],
            'records'   => [
                'request'  => 'http://127.0.0.1:80/admin/pt1coutpanel/playback?view=%s',
                'result'   => '',
                'login'    => '',
                'password' => '',
            ],
            'long_poll' => [
                'event_time_to_live' => 10,
            ],
            'files'     => $this->dirs['filesDir'],
        ];

        file_put_contents("{$this->dirs['confDir']}/ami.json",
            json_encode($settings_amid, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }

    /**
     * Создает файл настроек автоподъема трубки
     */
    private function generateHeadersConf()
    {

        $settings_headers = [
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Call-Info:\\;answer-after=0"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Call-Info)="\\;answer-after=0"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'linksys',
                    'cisco',
                    'miko',
                    'telephone-pt1c',
                    'nightbird',
                    'grandstream',
                    'microsip',
                    'zoiper',
                ],
            ],
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Call-Info:answer-after=0"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Call-Info)="answer-after=0"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'yealink',
                    'vp530p',
                ],
            ],
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Call-Info: sip:127.0.0.1\\;answer-after=0"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Call-Info)="sip:127.0.0.1\\;answer-after=0"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'snom',
                ],
            ],
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Alert-Info: info=alert-autoanswer"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Alert-Info)="info=alert-autoanswer"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'aastra',
                    'fanvil',
                ],
            ],
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Alert-Info: Ring Answer"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Alert-Info)="Ring Answer"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'polycom',
                ],
            ],
            [
                'header' => [
                    'default' => 'SIPADDHEADER="Alert-Info:Auto Answer"',
                    'pbx'     => [
                        [
                            'name'   => 'FreePBX',
                            'driver' => [
                                'PJSIP' => 'PJSIP_HEADER(add,Alert-Info)="Auto Answer"',
                            ],
                        ],
                    ],
                ],
                'phones' => [
                    'jitsi',
                ],
            ],
        ];

        file_put_contents("{$this->dirs['confDir']}/headers.json",
            json_encode($settings_headers, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }

    /**
     * Create configs for all files
     */
    public function generateConfFiles(){
        $this->generateNatsConf();
        $this->generateHeadersConf();
        $this->generateCrmdConf();
        $this->generateAuthdConf();
        $this->generateAmidConf();
        $this->generateMonitordConf();
    }

    /**
     * Ротация логов для службы NATS
     */
    public function logRotateGnats()
    {
        if ($this->module_settings['enable'] === '0') {
            return;
        }
        $log_dir  = "{$this->dirs['logDir']}/gnats";
        $pid_file = "{$this->dirs['pidDir']}/gnatsd-cti.pid";
        if (file_exists($pid_file)) {
            $pid = file_get_contents($pid_file);
        } else {
            return;
        }

        $max_size = 1;
        if (empty($pid)) {
            return;
        }
        $text_config = "{$log_dir}/gnatsd.log {
    start 0
    rotate 9
    size {$max_size}M
    maxsize 1M
    missingok
    notifempty
    sharedscripts
    postrotate
        {$this->dirs['binDir']}/gnatsd-cti -sl reopen={$pid} > /dev/null 2> /dev/null
    endscript
}";

        $mb10 = $max_size * 1024 * 1024;

        $options = '';
        if (Util::m_file_sise("{$log_dir}/gnatsd.log") > $mb10) {
            $options = '-f';
        }

        $path_conf = "{$this->dirs['confDir']}/gnatsd_cti_logrotate.conf";
        file_put_contents($path_conf, $text_config);
        $path_logrotate = fpbx_which('logrotate');
        if (file_exists("{$log_dir}/gnatsd.log")) {
            Util::mwexec("$path_logrotate $options '{$path_conf}' > /dev/null 2> /dev/null");
        }

        // Очистка логов AMI.
        $path_find  = fpbx_which('find');
        $path_rm    = fpbx_which('rm');
        $path_xargs = fpbx_which('xargs');
        Util::mwexec("$path_find '{$log_dir}' -name '*.log.[0-9]' -mtime +7 | $path_xargs $path_rm > /dev/null 2> /dev/null");
        Util::mwexec("$path_find '{$log_dir}' -name '*.log.[0-9][0-9]' -mtime +7 | $path_xargs $path_rm > /dev/null 2> /dev/null");
        Util::mwexec("$path_find '{$log_dir}' -name '*.log' -mtime +7 | $path_xargs $path_rm > /dev/null 2> /dev/null");
    }

    /**
     * Добавляет задания в CRON,
     * проверить можно
     * для FreePBX < 14 crontab -u asterisk -l
     * для FreePBX > 14 версии fwconsole job --list
     */
    public function updateCronTasks()
    {
        $path_php          = fpbx_which('php');
        $safeScriptCommand = "{$path_php} -f {$this->dirs['binDir']}/pt1coutpanel_safe_script.php > /dev/null 2>&1";
        $logRotateCommand  = "{$path_php} -f {$this->dirs['binDir']}/pt1coutpanel_logrotate.php > /dev/null 2>&1";

        if (version_compare_freepbx(getVersion(), '14.0', '>=')) {
            FreePBX::Job()->removeAllByModule('pt1coutpanel');
            FreePBX::Job()->addCommand('pt1coutpanel', 'safe_script', $safeScriptCommand, '*/3 * * * *');
            FreePBX::Job()->addCommand('pt1coutpanel', 'logrotate', $logRotateCommand, '*/30 * * * *');
        } else {
            foreach (FreePBX::Cron()->getAll() as $cron) {
                $str = 'pt1coutpanel_safe_script.php';
                if (preg_match('/' . $str . '/i', $cron, $matches)) {
                    FreePBX::Cron()->remove($cron);
                }
                $str = 'pt1coutpanel_logrotate.php';
                if (preg_match('/' . $str . '/i', $cron, $matches)) {
                    FreePBX::Cron()->remove($cron);
                }
            }
            if ($this->module_settings['enable'] === '1') {
                // Скрипт - провяряющий состояние демонов.
                FreePBX::Cron()->add(
                    [
                        'command' => $safeScriptCommand,
                        'hour'    => '*',
                        'minute'  => '*/3',
                    ]);
                // Сброс логов.
                FreePBX::Cron()->add(
                    [
                        'command' => $logRotateCommand,
                        'hour'    => '*',
                        'minute'  => '*/30',
                    ]);
            }
        }
    }

    /**
     * Запуск сервисов модуля
     */
    public function startAllServices($restart=false)
    {
        $this->generateConfFiles();
        $moduleEnabled = ($this->module_settings['enable'] === '1');
        // NATS работает постоянно, он отвечает за лицензирование и регистрацию модуля
        $nats_process_log = $this->dirs['logDir'] . '/gnats_process.log';
        $nats             = $this->dirs['binDir'] . '/gnatsd-cti';
        Util::process_worker($nats, "--config {$this->dirs['confDir']}/nats.conf", 'gnatsd-cti', 'start', $nats_process_log,
            $this->webuser);

        $monitord_process_log = $this->dirs['logDir'] . '/monitord_process.log';
        $monitord             = $this->dirs['binDir'] . '/monitord';
        $amid                 = $this->dirs['binDir'] . '/amid';
        $authd                = $this->dirs['binDir'] . '/authd';
        $crmd                 = $this->dirs['binDir'] . '/crmd';

        if ($moduleEnabled) {
            if ($restart){
                Util::process_worker($monitord, '', 'monitord', 'stop', $monitord_process_log);
                Util::process_worker($amid, '', 'amid', 'stop');
                Util::process_worker($authd, '', 'authd', 'stop');
                Util::process_worker($crmd, '', 'crmd', 'stop');

            }
            Util::process_worker($monitord, "-c {$this->dirs['confDir']}/monitord.json", 'monitord', 'start',
                $monitord_process_log, $this->webuser);
        } else {
            Util::process_worker($monitord, '', 'monitord', 'stop', $monitord_process_log);
            Util::process_worker($amid, '', 'amid', 'stop');
            Util::process_worker($authd, '', 'authd', 'stop');
            Util::process_worker($crmd, '', 'crmd', 'stop');
        }
    }

    /**
     * Тестирование живой ли модуль, доступны ли сервисы
     *
     * @return array
     */
    public function test()
    {
        $statusNats     = $this->checkNatsStatus();
        $status1C       = $this->checkWorkerStatus('1c');
        $statusAuth     = $this->checkWorkerStatus('auth');
        $statusAsterisk = $this->checkWorkerStatus('asterisk');

        $noError = $status1C['state'] === 'ok'
            && $statusAuth['state'] === 'ok'
            && $statusAsterisk['state'] === 'ok'
            && $statusNats['state'] === 'ok';

        return [
            'result'   => $noError,
            'statuses' => [
                $status1C,
                $statusAuth,
                $statusAsterisk,
                $statusNats,
            ],
        ];
    }

    /**
     * Проверка, запущена ли служба NATS
     *
     * @return array
     */
    private function checkNatsStatus()
    {
        $statusUrl = 'http://127.0.0.1:8222/varz';
        $curl      = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 1);
        curl_setopt($curl, CURLOPT_URL, $statusUrl);

        try {
            $responce = curl_exec($curl);
            $responce = str_replace('\n', '', $responce);
        } catch (Exception $e) {
            $responce = null;
        }
        $data = json_decode($responce, true);
        curl_close($curl);
        if ($data !== null) {
            $result = [
                'name'    => 'nats',
                'state'   => 'ok',
                'version' => $data['version'],
                'uptime'  => $data['uptime'],
                'start'   => $data['start'],
            ];
        } else {
            $result = [
                'name'    => 'nats',
                'state'   => 'unknown',
                'version' => 'unknown',
            ];
        }

        return $result;
    }

    /**
     * Возвращает статус воркера
     *
     * @param $workerName - название воркера, статус которого получаем
     *
     * @return array
     */
    private function checkWorkerStatus($workerName)
    {

        $statusUrl = 'http://127.0.0.1:8222/manager.api/status?daemon=' . $workerName;
        $curl      = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 1);
        curl_setopt($curl, CURLOPT_URL, $statusUrl);

        try {
            $responce = curl_exec($curl);
            $responce = str_replace('\n', '', $responce);
        } catch (Exception $e) {
            $responce = null;
        }
        $data = json_decode($responce, true);
        curl_close($curl);
        if ($data !== null
            && array_key_exists('result', $data)
            && is_array($data['result'])
        ) {
            $result = $data['result'];
        } else {
            $result = [
                'name'  => $workerName,
                'state' => 'unknown',
            ];
        }

        return $result;
    }


}