<?php


namespace FreePBX\modules\Pt1coutpanel\lib;


class Util
{
    /**
     * Управление процессом / демоном.
     * Получние информации по статусу процесса.
     * @param $cmd
     * @param $param
     * @param $proc_name
     * @param $action
     * @param $out_file
     * @param $user
     * @return array | bool
     */
    public static function process_worker($cmd, $param, $proc_name, $action, $out_file = '/dev/null', $user = null)
    {
        $path_kill  = fpbx_which('kill');
        $path_nohup = fpbx_which('nohup');

        if(!empty($param)){
            $proc_str = $cmd.' '.trim($param);
        }else{
            $proc_str = $proc_name;
        }
        $WorkerPID = self::get_pid_process($proc_str);

        if('status' === $action ){
            $status = ($WorkerPID !== '') ? 'Started' : 'Stoped';
            return array('status' => $status, 'app' => $proc_name, 'PID' => $WorkerPID);
        }
        $out = array();
        if('$WorkerPID' !== '' && ('stop' === $action || 'restart' === $action) ){
            self::mwexec("$path_kill -9 $WorkerPID  > /dev/null 2>&1 &", $out,$out_file, $user);
            $WorkerPID = '';
        }

        if($WorkerPID === '' && ('start' === $action || 'restart' === $action) ){
            self::mwexec("$path_nohup $cmd $param  > $out_file 2>&1 &", $out, $out_file, $user);
            usleep(500000);
        }

        return true;
    }

    /**
     * Возвращает PID процесса по его имени.
     * @param        $name
     * @param string $exclude
     * @return string
     */
    public static function get_pid_process($name, $exclude='')
    {
        $path_ps    = fpbx_which('ps');
        $path_grep  = fpbx_which('grep');
        $path_awk 	= fpbx_which('awk');

        $filter_cmd = '';
        if(!empty($exclude)){
            $filter_cmd = "| $path_grep -v ".escapeshellarg($exclude);
        }

        $out = array();
        self::mwexec("$path_ps -A -o 'pid,args' {$filter_cmd} | $path_grep '$name' | $path_grep -v grep | $path_awk ' {print $1} '", $out);
        return trim(implode(' ', $out));
    }

    /**
     * Выполняет системную команду exec().
     * @param      $command
     * @param null $oarr
     * @param null $retval
     * @param null $user
     * @return int|null
     */
    static function mwexec($command, &$oarr = null, &$retval=null, $user=null)
    {
        $retval = 0;
        $oarr	= array();
        if ($user !== null && posix_getuid()===0){
             exec("su {$user} -c {$command}", $oarr, $retval);
        } else {
             exec($command, $oarr, $retval);
        }

        return $retval;
    }

    /**
     * Завершаем процесс по имени.
     * @param $procname
     * @return int|null
     */
    static function killbyname($procname) {
        return Util::mwexec('killall '.escapeshellarg($procname));
    }

    /**
     *  Возвращает размер файла в Мб.
     * @param $filename
     * @return float|int
     */
    static function m_file_sise($filename){
        $size = 0;
        if(file_exists($filename)){
            $tmp_size = filesize($filename);
            if($tmp_size !== FALSE){
                // Получим размер в Мб.
                $size = $tmp_size;
            }
        }
        return $size;
    }

    /**
     * Создает папку и подправляет к ней права
     * @param $path
     * @param $user
     * @param $group
     *
     * @throws \Exception
     */
    static function createFolder($path, $user, $group){
        if ( ! file_exists($path) && ! mkdir($path, 0777, true) && ! is_dir($path)) {
            throw new \Exception(sprintf(_("Dir (%s) doesn't exist. Aborting"),
                $path));
        } else {
            chown($path, $user);
            chgrp($path, $group);
        }
    }
}